import ScTrackingService from './tracking.service';
import { Event } from './event.model';
import './index';
import 'solution-center-communicator';

import IInjectorService = angular.auto.IInjectorService;
import IProvideService = angular.auto.IProvideService;

describe('ScTrackingService', () => {
  let $httpBackend: ng.IHttpBackendService;
  let $rootScope: ng.IRootScopeService;
  let scTrackingService: ScTrackingService;
  let mock: any;
  let result: boolean;

  beforeEach(setup);
  afterEach(teardown);

  it('should call the api for a valid event', () => {
    result = false;
    $httpBackend.expectPOST(`${mock.endpoint}${mock.validEvent.eventType}`).respond(200);
    scTrackingService.track(mock.validEvent.eventType, mock.validEvent.eventBody).then(mock.success, mock.error);
    $rootScope.$digest();
    $httpBackend.flush();
    expect(result).toBe(true);
  });

  it('should reject an invalid event', () => {
    result = true;
    scTrackingService.track(mock.invalidEvent.eventType, mock.invalidEvent.eventBody).then(mock.success, mock.error);
    $rootScope.$digest();
    expect(ScTrackingService.eventValid).toHaveBeenCalled();
    expect(result).toBe(false);
  });

  it('should reject an event without event type', () => {
    result = true;
    scTrackingService.track(mock.noEventType.eventType, mock.noEventType.eventBody).then(mock.success, mock.error);
    $rootScope.$digest();
    expect(ScTrackingService.eventValid).not.toHaveBeenCalled();
    expect(result).toBe(false);
  });

  describe('eventValid', () => {
    it('should return true if all required field are set', () => {
      expect(ScTrackingService.eventValid(new Event(mock.validEvent.eventBody))).toBe(true);
    });
    it('should return false if any required fields are missing', () => {
      expect(ScTrackingService.eventValid(new Event(mock.invalidEvent.eventBody))).toBe(false);
    });
  });

  /////////////////////////

  function setup(): void {
    mocks();
    spies();
    modules();
    injectors();
  }

  function modules(): void {
    angular.mock.module('solutioncenter.tracking', ($provide: IProvideService) => {
      $provide.value('ScEnvironments', mock.environmentsProvider);
    });
  }

  function injectors(): void {
    angular.mock.inject(($injector: IInjectorService) => {
      $httpBackend = $injector.get('$httpBackend');
      $rootScope = $injector.get('$rootScope');
      scTrackingService = (<ScTrackingService>$injector.get('ScTrackingService'));
    });
  }

  function mocks(): void {
    mock = {
      validEvent: {
        eventType: 'USER_CREATED',
        eventBody: {
          product: 'Solution Center',
          component: 'Backend'
        }
      },
      invalidEvent: {
        eventType: 'USER_CREATED',
        eventBody: {
          source: 'Brand Selection Page',
          component: 'Backend'
        }
      },
      noEventType: {
        eventType: '',
        eventBody: {
          product: 'Solution Center',
          component: 'Backend'
        }
      },
      success: () => {
        result = true;
      },
      error: () => {
        result = false;
      },
      environmentsProvider: jasmine.createSpyObj('ScEnvironments', ['getCurrentEnvironment']),
      environment: { USER_SERVICE: 'mock.user.service' },
      endpoint: `mock.user.service${ScTrackingService.eventEndpoint}`,
    };
  }

  function spies(): void {
    spyOn(ScTrackingService, 'eventValid').and.callThrough();
    return mock.environmentsProvider.getCurrentEnvironment.and.returnValue(mock.environment);
  }

  function teardown(): void {
    $httpBackend.verifyNoOutstandingExpectation();
    $httpBackend.verifyNoOutstandingRequest();
  }

});
