import { Event, EventInterface } from './event.model';
import TrackingError from './tracking-error.model';
import ScEnvironmentsProvider = ScCommunicator.ScEnvironmentsProvider;

class ScTrackingService {
  static $inject: Array<string> = ['$http', '$q', 'ScEnvironments'];
  static eventEndpoint: string = '/events/business-events/';

  private eventUrl: string;

  static eventValid(event: Event): boolean {
    return (!!event.product && !!event.component);
  }

  constructor(private $http: ng.IHttpService,
              private $q: ng.IQService,
              private ScEnvironments: ScEnvironmentsProvider) {
    this.eventUrl = ScEnvironments.getCurrentEnvironment().USER_SERVICE + ScTrackingService.eventEndpoint;
  }

  private constructUrl(eventType: string): string {
    return `${this.eventUrl}${eventType}`;
  }

  track(eventType: string, e: EventInterface): ng.IPromise<any> {
    if (!eventType) {
      return this.$q.reject(new TrackingError(TrackingError.NO_EVENT_TYPE));
    }

    let event = new Event(e);

    if (ScTrackingService.eventValid(event)) {
      return this.$http.post(this.constructUrl(eventType), event);
    } else {
      return this.$q.reject(new TrackingError(TrackingError.INVALID_EVENT));
    }
  }
}

export default ScTrackingService;
